package com.extrieve.quickcapture;

import android.app.Activity;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.util.Log;
import android.widget.Toast;
import com.google.gson.Gson;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import io.flutter.embedding.android.FlutterActivity;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.extrieve.quickcapture.sdk.ImgHelper;
import com.extrieve.quickcapture.sdk.Config;
import com.extrieve.quickcapture.sdk.Config.CaptureSupport;

import java.io.IOException;
import java.util.ArrayList;

/** QuickcapturePlugin */
public class QuickcapturePlugin extends FlutterActivity implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private static ArrayList<String> captureimages = new ArrayList<>();
  private int REQUEST_CODE_PERMISSIONS = 1001;
  public int REQUEST_CODE_FILE_RETURN = 1002;
  private MethodChannel channel;
  private Activity activity;
  private Context context;
  private ImgHelper ImageHelper;
  private Result result;


  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "quickcapture");
    channel.setMethodCallHandler(this);
    context = flutterPluginBinding.getApplicationContext();

    try {
      ImageHelper = new ImgHelper(context);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
    try {
      ImageHelper.Init();
    } catch (Exception e) {
      throw new RuntimeException(e);
    }

  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    this.result = result;
    switch (call.method){
      case "initialize":
        CaptureSupport.EnableFlash = true;
        CaptureSupport.DocumentCropping = CaptureSupport.CroppingType.LiveDetection;
        result.success("QuickCapture initialised successfully");
        break;
      case "getPlatformVersion":
        result.success(Config.CaptureSupport.OutputPath);
        break;
      case "startCapture":
        Config.CaptureSupport.OutputPath = BuildStoragePath();
        this.result = result;
        OpenCameraActivity();
        break;
      default:
        result.notImplemented();
        break;
    }

  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding binding) {
    activity = binding.getActivity();
    binding.addActivityResultListener((requestCode, resultCode, data) -> {
      onActivityResult(requestCode, resultCode, data);
      return false;
    });
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    activity = binding.getActivity();
    binding.addActivityResultListener((requestCode, resultCode, data) -> {
      onActivityResult(requestCode, resultCode, data);
      return false;
    });
  }

  @Override
  public void onDetachedFromActivity() {
    activity = null;
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
    //binding.removeRequestPermissionsResultListener(this);
    //binding.removeActivityResultListener(this);
  }

  protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
    super.onActivityResult(requestCode, resultCode, data);

    if (requestCode == REQUEST_CODE_FILE_RETURN && resultCode == Activity.RESULT_OK) {
      Boolean Status = (Boolean) data.getExtras().get("STATUS");
      String Description = (String) data.getExtras().get("DESCRIPTION");
      if (Status == false) {
        String imageCaptureLog = "Description : " + Description +
                ".Exception: " + Config.CaptureSupport.LastLogInfo;
        Log.d("INFO", imageCaptureLog);
        activity.finishActivity(REQUEST_CODE_FILE_RETURN);
        return;
      }
      captureimages = (ArrayList<String>) data.getExtras().get("fileCollection");
      if (captureimages == null || captureimages.isEmpty()) return;
      OnActivitySuccess();
      //showImages(FileCollection);
      //activity.finishActivity(REQUEST_CODE_FILE_RETURN);
    }
  }

  class returnClass{
    ArrayList<String> fileCollection = null;
    String STATUS = "SUCCESS";
    String DESCRIPTION = "SUCCESS";
  }
  private void OnActivitySuccess() {
    Intent intent = new Intent();
    intent.putExtra("STATUS", true);
    intent.putExtra("DESCRIPTION", "");
    intent.putExtra("fileCollection", captureimages);
    returnClass tr = new returnClass();
    tr.fileCollection = captureimages;
    Gson gson = new Gson();
    String jsonString = gson.toJson(tr);
    result.success(jsonString);
    //channel.invokeMethod("_onActivityResult", null);
    //activity.setResult(RESULT_OK, intent);
    activity.finishActivity(REQUEST_CODE_FILE_RETURN);

    //finish();
  }

  private void OnActivityFail(String MethodName, String FailedDescription) {
    Intent intent = new Intent();
    intent.putExtra("STATUS", false);

    String desc = "Failure in Camera section : at " + MethodName;
    if (FailedDescription != null && !FailedDescription.isEmpty())
      desc += " " + FailedDescription;
    intent.putExtra("DESCRIPTION", desc);
    intent.putExtra("fileCollection", "");
    setResult(RESULT_OK, intent);
    finish();
  }

  private void OpenCameraActivity()  {
    //before starting camera - configuration can set
    //String quality = ImageHelper.getCurrentImageQuality();
    try {
      //moving to camera activity in library
      Intent CameraIntent = new Intent(activity, Class.forName("com.extrieve.quickcapture.sdk.CameraHelper"));
      //photoURI = CameraSupport.CamConfigClass.OutputPath;
      Uri photoURI = Uri.parse(Config.CaptureSupport.OutputPath);
      activity.grantUriPermission(activity.getPackageName(), photoURI,
              Intent.FLAG_GRANT_WRITE_URI_PERMISSION | Intent.FLAG_GRANT_READ_URI_PERMISSION);
      if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.LOLLIPOP) {
        CameraIntent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
      }
      activity.startActivityForResult(CameraIntent, REQUEST_CODE_FILE_RETURN);
      //someActivityResultLauncher.launch(CameraIntent);

    } catch (ClassNotFoundException e) {
      Toast.makeText(activity, "Failed to open camera + ", Toast.LENGTH_LONG).show();
      e.printStackTrace();
    }
  }



  @NonNull
  private String BuildStoragePath() {
    //File direct = new File(Environment.getExternalStorageDirectory(), ".GoNoGoImages");
    //return direct.getAbsolutePath();
    if(activity == null) return null;
    ContextWrapper c = new ContextWrapper(activity);
    String path = c.getExternalFilesDir(".QCImages").getAbsolutePath();
    return path;
  }
}


